import java.net.*;

/**
 * Klasse fuer einen SpielServer
 * @author Henning Ainödhofer
 * @version 21.03.2009
 */

public class SpielServer extends Server {

    private HighscoreGateway DBhighscore;
    private SpielerGateway DBspieler;
    private List<Spiel> spieleOnline;
    
    public SpielServer(int p) {
        super(p);
        DBhighscore = new HighscoreGateway();
        DBspieler = new SpielerGateway();
        spieleOnline = new List<>();
    }

    /**
     * Diese Methode der Server-Klasse wird hiermit ueberschrieben.
     * Der angemeldete Client bekommt die Meldung, dass er angenommen wurde.
     */

    public void processNewConnection(String pClientIP, int pClientPort){
        this.send(pClientIP, pClientPort, "CON complete");
        spieleOnline.append(new Spiel(pClientIP, pClientPort, gibZufallszahl()));
    }

    /**
     * Diese Methode der Server-Klasse wird hiermit ueberschrieben.
     * Der angemeldete Client bekommt die gesendete Meldung zurueckgeschickt.
     */

    public void processMessage(String pClientIP, int pClientPort, String pMessage){
        String[] m = pMessage.split(":"); 
        switch(m[0])
        {
            case "PLY": //Spielen
            {
                this.send(pClientIP, pClientPort, m[1]);
                break;
            }
            case "SND": //Zahl gesendet
            {
                if(gibZahlVonSpiel(pClientIP) == Integer.parseInt(m[1]))
                {
                    this.send(pClientIP, pClientPort, "POS:Sie haben nach "+gibVersucheVonSpiel(pClientIP)+" Versuchen richtig geraten!");
                }
                else
                {
                    this.send(pClientIP, pClientPort, "NEG:Leider falsch, raten Sie weiter");
                }
                break;
            }
            case "LOG": //Anmelden mit Benutzername und Passwort
            {
                
                break;
            }
            case "REG": //Neu Registrieren
            {
                
                break;
            }
            case "HSC": //Highscoreliste mit den 10 Besten zurückgeben
            {
                
                break;
            }
            case "END": // Verbindung beenden
            {
                
                break;
            }
            default:
            {
                this.send(pClientIP, pClientPort, "ERR01 Befehl nicht bekannt");
                break;
            }
        }

    }

    /**
     * Diese Methode der Server-Klasse wird hiermit ueberschrieben.
     * Die Verbindung wird beendet und aus der Liste der Clients gestrichen.
     */

    public void processCloseConnection(String pClientIP, int pClientPort){
        this.send(pClientIP, pClientPort, "EXT complete");
        this.closeConnection(pClientIP, pClientPort);
    }
    
    public static void main(String [] args)
    {
        SpielServer es = new SpielServer(1024);
    }

    private synchronized int gibZufallszahl()
    {
        return (int)(Math.random() * 20);
    }
    
    private synchronized int gibZahlVonSpiel(String pClientIP)
    {
        Spiel erg = null;
        spieleOnline.toFirst();
        while(spieleOnline.hasAccess())
        {
            if(spieleOnline.getContent().gibClientIP().equals(pClientIP))
            {
                erg = spieleOnline.getContent();
                break;
            }
            spieleOnline.next();
        }
        return erg.gibZahl();
    }
    
    private synchronized int gibVersucheVonSpiel(String pClientIP)
    {
        int erg = 20;
        spieleOnline.toFirst();
        while(spieleOnline.hasAccess())
        {
            if(spieleOnline.getContent().gibClientIP().equals(pClientIP))
            {
                erg = spieleOnline.getContent().gibVersuche();
                break;
            }
            spieleOnline.next();
        }
        return erg;
    }
    
    private synchronized void versucheErhoehenVonSpiel(String pClientIP)
    {
        spieleOnline.toFirst();
        while(spieleOnline.hasAccess())
        {
            if(spieleOnline.getContent().gibClientIP().equals(pClientIP))
            {
                spieleOnline.getContent().erhoeheVeruche();
                break;
            }
            spieleOnline.next();
        }
    }
}
